SystemOrganization addCategory: #'WriteBarrier-Core'!
SystemOrganization addCategory: #'WriteBarrier-Tests'!

Object subclass: #WBMethodBuilder
	instanceVariableNames: 'selector class fields generator'
	classVariableNames: 'Cache'
	poolDictionaries: ''
	category: 'WriteBarrier-Core'!

!WBMethodBuilder methodsFor: 'as yet unclassified' stamp: 'avi 10/6/2004 12:09'!
generateCheckField: aNumber
	generator pushInstVar: aNumber.
	generator pushTemp: (self tempForField: aNumber).
	generator send: #==.
	generator if: false goto: #notify.! !

!WBMethodBuilder methodsFor: 'as yet unclassified' stamp: 'avi 10/6/2004 11:53'!
generateReturn 
	generator pushTemp: self returnValueTemp.
	generator returnTop! !

!WBMethodBuilder methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 03:28'!
buildCompiledMethod
	generator _ BytecodeGenerator new numArgs: selector numArgs.
	self generateStoreFields.
	self generateCallSuper.
	self generateCheckFields.
	self generateReturn.
	self generateNotification.
	self generateReturn.
	^ generator compiledMethod! !

!WBMethodBuilder methodsFor: 'as yet unclassified' stamp: 'avi 10/6/2004 11:49'!
generateCheckFields
	fields do: [:i | self generateCheckField: i]! !

!WBMethodBuilder class methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 03:28'!
initialize
	Cache _ WeakIdentityKeyDictionary new.
	WeakArray addWeakDependent: Cache! !

!WBMethodBuilder methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 03:15'!
generateNotification
	generator label: #notify.
	generator pushReceiver.
	generator send: #writeBarrier.
	generator pushReceiver.
	generator send: #modified:! !

!WBMethodBuilder class methodsFor: 'as yet unclassified' stamp: 'avi 10/6/2004 13:51'!
selector: aSymbol class: aClass fields: anArray
	^ self basicNew
		initializeWithSelector: aSymbol
		class: aClass
		fields: anArray! !

!WBMethodBuilder methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 03:28'!
compiledMethod
	^ Cache at: (class lookupSelector: selector) ifAbsentPut: [self buildCompiledMethod]! !

!WBMethodBuilder methodsFor: 'as yet unclassified' stamp: 'avi 10/6/2004 11:43'!
generateStoreField: aNumber
	generator pushInstVar: aNumber.
	generator storePopTemp: (self tempForField: aNumber)! !

!WBMethodBuilder methodsFor: 'as yet unclassified' stamp: 'avi 10/6/2004 11:42'!
generateStoreFields
	fields do: [:i | self generateStoreField: i]! !

!WBMethodBuilder methodsFor: 'as yet unclassified' stamp: 'avi 10/6/2004 11:48'!
returnValueTemp
	^ selector numArgs + fields size + 1! !

!WBMethodBuilder methodsFor: 'as yet unclassified' stamp: 'avi 10/6/2004 12:07'!
generateCallSuper
	generator pushReceiver.
	1 to: selector numArgs do: [:i | generator pushTemp: i].
	generator send: selector toSuperOf: class.
	generator storePopTemp: self returnValueTemp! !

!WBMethodBuilder methodsFor: 'as yet unclassified' stamp: 'avi 10/6/2004 11:44'!
tempForField: aNumber
	^ selector numArgs + (fields indexOf: aNumber)! !

!WBMethodBuilder methodsFor: 'as yet unclassified' stamp: 'avi 10/6/2004 13:51'!
initializeWithSelector: aSymbol class: aClass fields: anArray
	selector _ aSymbol.
	class _ aClass.
	fields _ anArray! !

Object subclass: #WriteBarrier
	instanceVariableNames: 'builders'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WriteBarrier-Core'!

!WriteBarrier commentStamp: '<historical>' prior: 0!
A WriteBarrier provides a way to watch for changes to the state of individual objects.  Any objects that you #add: to a WriteBarrier will be modified in the following ways:

- their class will be changed to be a new, anonymous subclass of whatever their class was before.  For example, if you #add: an Array to a WriteBarrier, it will now be of class "Array with write barrier", which is a subclass of Array (although it won't show up in "Array subclasses").
- any methods that potentially modify instance variables will be overriden in this new class.  The overridden method stores the original values of the instance variable in temps, then calls the super method, and then compares the current inst var values with the originals.
- If the instance variable values have changed, the WriteBarrier will be notified with a send to #modified:, with the object that was modified as the single argument.
- For variably-sized classes, #at:put: is also overridden to provide the same notification.
- the new class will override #isBehindWriteBarrier to return true, and will implement #writeBarrier to return the WriteBarrier instance.

The default implementation of #modified: doesn't do anything.  DirtySetWriteBarrier is a subclass that overrides #modified: to maintain a set of objects that have recently changed.  You may wish to create your own subclass with an alternate implementation.

Notes:
- You can only add any given object to one WriteBarrier.  If you try to add it to a second one, you will get an error.
- The classes of any objects you add to a WriteBarrier will be made uncompact.  For most compact classes (Array, String, Point, etc) the only effect this should have is to increase your image size by a few percent.  For some classes (CompiledMethod, MethodContext), the results are less predictable, and so adding them to a WriteBarrier is not supported.
- Because it's special cased by the VM, sending #class to an object will return the special anonymous class.  However, sending #species will still return the normal class (or whatever #species would have returned normally).
- The WriteBarrier is semi-permeable: if you want to modify an object directly without triggering notifications, you can use #instVarAt:put: and #basicAt:put:.!

!WriteBarrier methodsFor: 'as yet unclassified' stamp: 'avi 10/6/2004 13:40'!
registerForNotifications
	SystemChangeNotifier uniqueInstance notify: self ofAllSystemChangesUsing: #systemChanged:! !

!WriteBarrier methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 03:18'!
storeOn: aStream
	aStream nextPutAll: 'self writeBarrier'! !

!WriteBarrier methodsFor: 'as yet unclassified' stamp: 'avi 10/6/2004 13:45'!
modified: anObject! !

!WriteBarrier methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 02:15'!
systemChanged: anEvent
	builders keysAndValuesDo:
		[:class :builder |
		(class withAllSuperclasses includes: anEvent itemClass) ifTrue:
			[builder systemChanged: anEvent]]! !

!WriteBarrier methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 11:57'!
initialize
	builders _ WeakIdentityKeyDictionary new.
	WeakArray addWeakDependent: builders.
	self registerForNotifications ! !

!WriteBarrier methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 03:14'!
add: anObject
	anObject isBehindWriteBarrier
		ifTrue: [anObject writeBarrier = self ifFalse:
					[self error: 'This object is already behind another write barrier.']]
		ifFalse:
			[anObject primitiveChangeClassTo:
				(self barrierClassForClass: anObject class) basicNew]! !

!WriteBarrier methodsFor: 'as yet unclassified' stamp: 'avi 10/6/2004 13:25'!
barrierClassForClass: aClass
	^ (self builderForClass: aClass) barrierClass! !

!WriteBarrier methodsFor: 'as yet unclassified' stamp: 'avi 10/6/2004 13:22'!
builderForClass: aClass
	^ builders at: aClass ifAbsentPut: [self newBuilderForClass: aClass]! !

WriteBarrier subclass: #DirtySetWriteBarrier
	instanceVariableNames: 'dirty'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WriteBarrier-Tests'!

!DirtySetWriteBarrier methodsFor: 'as yet unclassified' stamp: 'avi 10/6/2004 13:58'!
dirtySet
	^ dirty copy! !

!DirtySetWriteBarrier methodsFor: 'as yet unclassified' stamp: 'avi 10/6/2004 13:58'!
isDirty: anObject
	^ dirty includes: anObject! !

!DirtySetWriteBarrier methodsFor: 'as yet unclassified' stamp: 'avi 10/6/2004 13:58'!
modified: anObject
	dirty add: anObject! !

!DirtySetWriteBarrier methodsFor: 'as yet unclassified' stamp: 'avi 10/6/2004 13:58'!
clearDirtySet
	dirty _ IdentitySet new! !

!DirtySetWriteBarrier methodsFor: 'as yet unclassified' stamp: 'avi 10/6/2004 13:58'!
initialize
	super initialize.
	self clearDirtySet! !

!WriteBarrier methodsFor: 'as yet unclassified' stamp: 'avi 10/6/2004 13:56'!
unregister
	SystemChangeNotifier uniqueInstance noMoreNotificationsFor: self! !

!WriteBarrier methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 13:11'!
remove: anObject
	anObject isBehindWriteBarrier ifTrue:
		[anObject writeBarrier == self ifFalse: [^ self error: 'This object is behind a different write barrier.'].
		anObject primitiveChangeClassTo: anObject class superclass basicNew]! !

!WriteBarrier methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 03:11'!
newBuilderForClass: aClass
	^ WBClassBuilder target: aClass barrier: self! !

Object subclass: #WBMock
	instanceVariableNames: 'x y'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WriteBarrier-Tests'!

WBMock subclass: #WBMockSub
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WriteBarrier-Tests'!

!WBMock methodsFor: 'as yet unclassified' stamp: 'avi 10/6/2004 12:20'!
y: anObject
	y _ anObject! !

!WBMock methodsFor: 'as yet unclassified' stamp: 'avi 10/6/2004 12:21'!
lazyX
	^ x ifNil: [x _ 42]! !

!WBMock methodsFor: 'tests' stamp: 'avi 10/7/2004 13:54'!
y ^ y! !

!WBMock methodsFor: 'tests' stamp: 'avi 10/7/2004 13:54'!
x: n x _ n! !

Object subclass: #WBClassBuilder
	instanceVariableNames: 'class barrier fieldsCache'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WriteBarrier-Core'!

!WBClassBuilder methodsFor: 'as yet unclassified' stamp: 'avi 10/6/2004 13:17'!
shouldOverrideAtPut
	^ class isVariable! !

!WBClassBuilder methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 03:06'!
systemChanged: anEvent
	anEvent itemKind = #method ifTrue:
		[(anEvent isRemoved and: [class selectors includes: anEvent itemSelector])
			ifTrue: [class removeSelectorSimply: anEvent itemSelector].
		self checkOverrideOf: anEvent itemSelector].
	anEvent itemKind = #class ifTrue:
		[anEvent isModified ifTrue:
			[self checkClassFormat: anEvent.
			self rebuildMethods]].! !

!WBClassBuilder methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 11:46'!
allSelectorsDo: aBlock
	((self targetClass withAllSuperclasses copyWithout: Object) copyWithout: ProtoObject) do:
		[:cls |
		cls selectors do: aBlock]! !

!WBClassBuilder methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 03:13'!
installTestingOverrides
	class
		compileSilently: 'isBehindWriteBarrier ^ true'
		classified: 'writebarrier'! !

!WBClassBuilder methodsFor: 'as yet unclassified' stamp: 'avi 10/6/2004 14:09'!
createSubclassOf: aClass
	| meta |
	meta _ Metaclass new.
	meta
		superclass: aClass class
		methodDictionary: MethodDictionary new
		format: aClass class format.
	class _ meta new.
	class
		superclass: aClass
		methodDictionary: MethodDictionary new
		format: aClass format.
	class setName: aClass name, ' with write barrier'.
	^ class
! !

!WBClassBuilder methodsFor: 'as yet unclassified' stamp: 'avi 10/6/2004 14:25'!
installOverrides
	self installStandardOverrides.
	self installFieldOverrides.  ! !

!WBClassBuilder methodsFor: 'as yet unclassified' stamp: 'avi 10/6/2004 01:33'!
targetClass
	^ class superclass! !

!WBClassBuilder methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 12:01'!
checkClassFormat: anEvent
	((class format ~~ class superclass format) or: [anEvent areInstVarsModified]) ifTrue:
		[self migrateFromTarget: anEvent oldItem to: anEvent itemClass]! !

!WBClassBuilder methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 13:37'!
installAccessingOverrides
	class addSelector: #writeBarrier withMethod:
		(BytecodeGenerator new
			numArgs: 0;
			pushLiteral: (WeakArray with: barrier);
			send: #first;
			returnTop;
			compiledMethod).
	class compileSilently:
		'species
		|species|
		species _ super species.
		^ species = self class
			ifTrue: [self class superclass]
			ifFalse: [species]'
		classified: 'accessing'! !

!WBClassBuilder methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 11:46'!
installFieldOverrides
	| fields |
	self allSelectorsDo:
		[:ea |
		fields _ self targetClass fieldsWrittenToBySelector: ea.
		fields isEmpty ifFalse: [self installOverrideOf: ea fields: fields]]! !

!WBClassBuilder methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 03:05'!
migrateFromTarget: oldSuperclass to: newSuperclass
	| oldClass |
	oldClass _ class.
	oldClass superclass: oldSuperclass.
	class _ self createSubclassOf: newSuperclass.
	class updateInstancesFrom: oldClass! !

!WBClassBuilder methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 02:28'!
checkOverrideOf: aSymbol
	| fields |
	fields _ self targetClass fieldsWrittenToBySelector: aSymbol.
	fields isEmpty 
		ifTrue: [(class selectors includes: aSymbol)
					ifTrue: [class removeSelectorSimply: aSymbol]]
		ifFalse: [(fields = (fieldsCache at: aSymbol ifAbsent: []))
					ifFalse: [self installOverrideOf: aSymbol fields: fields]]! !

!WBClassBuilder methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 03:12'!
installStandardOverrides
	self installAtPutOverrides.
	self installTestingOverrides.
	self installAccessingOverrides! !

!WBClassBuilder methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 11:46'!
rebuildMethods
	self allSelectorsDo: [:ea | self checkOverrideOf: ea]! !

!WBClassBuilder class methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 03:11'!
target: aClass barrier: aBarrier
	^ self basicNew initializeWithTarget: aClass barrier: aBarrier! !

!WBClassBuilder methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 03:15'!
installAtPutOverrides
	self shouldOverrideAtPut ifTrue:
	[class compileSilently: 
	'at: aNumber put: anObject
	super at: aNumber put: anObject.
	self writeBarrier modified: self'
	classified: 'standard']! !

!WBClassBuilder methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 03:11'!
initializeWithTarget: aClass barrier: aBarrier
	aClass ensureUncompact.
	barrier _ aBarrier.
	class _ self createSubclassOf: aClass.
	fieldsCache _ Dictionary new.
	self installOverrides.! !

!WBClassBuilder methodsFor: 'as yet unclassified' stamp: 'avi 10/6/2004 12:17'!
barrierClass
	^ class! !

!WBClassBuilder methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 02:23'!
installOverrideOf: aSymbol fields: anArray
	class addSelectorSilently: aSymbol withMethod:
		(WBMethodBuilder selector: aSymbol class: class fields: anArray)
			compiledMethod.
	fieldsCache at: aSymbol put: anArray! !

!Object methodsFor: '*writebarrier' stamp: 'avi 10/7/2004 03:13'!
isBehindWriteBarrier
	^ false ! !

!PseudoContext class methodsFor: '*writebarrier' stamp: 'avi 10/7/2004 13:30'!
canUncompact
	^ false! !

!CompiledMethod class methodsFor: '*writebarrier' stamp: 'avi 10/7/2004 13:29'!
canUncompact
	^ false! !

!Behavior methodsFor: '*writebarrier' stamp: 'avi 10/6/2004 12:57'!
canUncompact
	^ true! !

!Behavior methodsFor: '*writebarrier' stamp: 'avi 10/6/2004 12:57'!
ensureUncompact
	self indexIfCompact > 0 ifTrue:
		[self canUncompact
			ifTrue: [self becomeUncompact]
			ifFalse: [self error: 'Cannot uncompact ', self name]]! !

!Behavior methodsFor: '*writebarrier' stamp: 'avi 10/7/2004 02:23'!
fieldsWrittenToBySelector: aSymbol
	| cm |
	cm _ self lookupSelector: aSymbol.
	^ cm ifNil: [#()] ifNotNil: [(1 to: self instSize) select: [:i | cm writesField: i]]! !

!Behavior methodsFor: '*writebarrier' stamp: 'avi 10/6/2004 12:06'!
holder
	^ #holder -> self! !

!ContextPart class methodsFor: '*writebarrier' stamp: 'avi 10/7/2004 13:30'!
canUncompact
	^ false! !

TestCase subclass: #WBTestBarrier
	instanceVariableNames: 'barrier'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WriteBarrier-Tests'!

!WBTestBarrier methodsFor: 'as yet unclassified' stamp: 'avi 10/6/2004 12:14'!
testClassChangeCompact
	| point |
	point _ 2@2.
	self assert: point class == Point.
	self enableWriteBarrier: point.
	self deny: point class  == Point.! !

!WBTestBarrier methodsFor: 'as yet unclassified' stamp: 'avi 10/6/2004 14:02'!
testTrackIndividualInstances
	| point point2 point3 |
	point _ 2@2.
	point2 _ 2@2.
	point3 _ 2@2.
	self enableWriteBarrier: point.
	self enableWriteBarrier: point2.
	point setX: 25 setY: 25.
	point3 setX: 25 setY: 25.
	self assertChanged: point.
	self denyChanged: point2.
	self denyChanged: point3! !

!WBTestBarrier methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 03:19'!
assertChanged: anObject
	self assert: (barrier isDirty: anObject)! !

!WBTestBarrier methodsFor: 'as yet unclassified' stamp: 'avi 10/6/2004 14:01'!
testSelectiveNotification
	| m |
	m _ WBMock new.
	self enableWriteBarrier: m.
	self denyChanged: m.
	m lazyX.
	self assertChanged: m.
	self resetChanged.
	m lazyX.
	self denyChanged: m! !

!WBTestBarrier methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 13:38'!
testSpecies
	| array interval |
	array _ #(1 2 3) copy.
	interval _ 1 to: 10.
	self assert: array species == Array.
	self assert: interval species == Array.
	self enableWriteBarrier: array.
	self enableWriteBarrier: interval.
	self assert: array species == Array.
	self assert: interval species == Array! !

!WBTestBarrier methodsFor: 'as yet unclassified' stamp: 'avi 10/6/2004 14:02'!
testWriteBarrier
	| point |
	point _ 2@2.
	self enableWriteBarrier: point.
	point setX: 25 setY: 25.
	self assertChanged: point! !

!WBTestBarrier methodsFor: 'as yet unclassified' stamp: 'avi 10/6/2004 13:06'!
testDoubleWriteBarrier
	| m |
	m _ WBMock new.
	self assert: m class == WBMock.
	self enableWriteBarrier: m.
	self assert: m class superclass == WBMock.
	self enableWriteBarrier: m.
	self assert: m class superclass == WBMock! !

!WBTestBarrier methodsFor: 'as yet unclassified' stamp: 'avi 10/6/2004 14:02'!
testNoWriteBarrier
	| point |
	point _ 2@2.
	point setX: 25 setY: 25.
	self denyChanged: point! !

!WBTestBarrier methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 03:19'!
denyChanged: anObject
	self deny: (barrier isDirty: anObject)! !

!WBTestBarrier methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 13:32'!
testCannotAddBlock
	self should: [self enableWriteBarrier: [2+2]] raise: Error! !

!WBTestBarrier methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 03:19'!
enableWriteBarrier: anObject
	barrier add: anObject! !

!WBTestBarrier methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 03:19'!
tearDown
	barrier unregister! !

!WBTestBarrier methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 03:14'!
testCompetingManager
	| manager2 x y |
	manager2 _ WriteBarrier new.
	x _ WBMock new.
	self shouldnt: [manager2 add: x] raise: Error.
	y _ WBMock new.
	self enableWriteBarrier: y.
	self should: [manager2 add: y] raise: Error
	! !

!WBTestBarrier methodsFor: 'as yet unclassified' stamp: 'avi 10/6/2004 12:14'!
testClassChange
	| dict |
	dict _ Dictionary new.
	self assert: dict class == Dictionary.
	self enableWriteBarrier: dict.
	self deny: dict class  == Dictionary.! !

!WBTestBarrier methodsFor: 'as yet unclassified' stamp: 'avi 10/6/2004 14:01'!
testAtPutOverriden
	| array |
	array _ ColorArray new: 3.
	array at: 1 put: Color red.
	self denyChanged: array.
	self enableWriteBarrier: array.
	array at: 2 put: Color blue.
	self assertChanged: array! !

!WBTestBarrier methodsFor: 'as yet unclassified' stamp: 'avi 10/6/2004 14:01'!
testAtPutVariablySized
	| array |
	array _ Array new: 3.
	array at: 1 put: 42.
	self denyChanged: array.
	self enableWriteBarrier: array.
	array at: 2 put: 37.
	self assertChanged: array! !

!WBTestBarrier methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 03:19'!
setUp
	barrier _ DirtySetWriteBarrier new! !

!WBTestBarrier methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 13:09'!
testRemove
	| point |
	point _ 2@2.
	self enableWriteBarrier: point.
	point setX: 25 setY: 25.
	self assertChanged: point.
	barrier remove: point.
	self resetChanged.
	point setX: 26 setY: 26.
	self denyChanged: point! !

!WBTestBarrier methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 03:19'!
resetChanged
	barrier clearDirtySet ! !

!WBTestBarrier methodsFor: 'as yet unclassified' stamp: 'avi 10/6/2004 14:01'!
testAtPutDelegatingCollection
	| coll |
	coll _ OrderedCollection new: 5.
	coll add: 'bar'.
	self enableWriteBarrier: coll.
	coll at: 1 put: 'foo'.
	self denyChanged: coll.
	1 to: 100 do: [:i | coll add: i].
	self assertChanged: coll! !

TestCase subclass: #WBTestSystemChanges
	instanceVariableNames: 'barrier'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WriteBarrier-Tests'!

!WBTestSystemChanges methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 03:20'!
setUp
	barrier _ WriteBarrier new! !

!WBTestSystemChanges methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 03:22'!
testClassFormatChange
	| oldFmt c |
	oldFmt _ WBMock format.
	c _ self barrierClass.
	self assert: c format = WBMock format.
	WBMock addInstVarName: 'foo'.
	c _ self barrierClass.
	[self deny: WBMock format = oldFmt.
	self assert: c format = WBMock format]
		ensure: [WBMock removeInstVarName: 'foo']! !

!WBTestSystemChanges methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 03:22'!
testAddMutation
	| c |
	c _ self barrierClass.
	self withDefaultMethod: 'y ^ y' on: WBMock do:
		[self deny: (c selectors includes: #y).
		WBMock compile: 'y ^ y ifNil: [y _ 37]' classified: 'accessing'.
		self assert: (c selectors includes: #y)]! !

!WBTestSystemChanges methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 03:22'!
testAddMutator
	| c md cm |
	c _ self barrierClass.
	md _ c methodDictionary copy.
	cm _ c >> #x:.
	self withTemporaryMethod: 'q ^ x _ 42' on: WBMock do:
		[self assert: (c >> #x:) == cm.
		self deny: md = c methodDictionary.
		self assert: (c selectors includes: #q)]! !

!WBTestSystemChanges methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 02:08'!
withTemporaryMethod: aString on: aClass do: aBlock
	aClass compile: aString classified: 'tests'.
	aBlock ensure: [aClass removeSelector: (Parser new parseSelector: aString)]! !

!WBTestSystemChanges methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 03:21'!
testChangeDifferentField
	| c cm |
	c _ self barrierClass.
	self withDefaultMethod: 'x: n x _ n' on: WBMock do:
		[cm _ c >> #x:.
		WBMock compile: 'x: n y _ n'.
		self deny: (c >> #x:) == cm]! !

!WBTestSystemChanges methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 03:21'!
barrierClass
	^ barrier barrierClassForClass: WBMock! !

!WBTestSystemChanges methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 03:22'!
testAddNonMutator
	| c md cm |
	c _ self barrierClass.
	md _ c methodDictionary copy.
	cm _ c >> #x:.
	self withTemporaryMethod: 'q ^ 42' on: WBMock do:
		[self assert: (c >> #x:) == cm.
		self assert: md = c methodDictionary.
		self deny: (c selectors includes: #q)]! !

!WBTestSystemChanges methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 11:47'!
tearDown
	barrier unregister! !

!WBTestSystemChanges methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 03:22'!
testRemoveMutator
	| c |
	c _ self barrierClass.
	self withDefaultMethod: 'x: n x _ n' on: WBMock do:
		[self assert: (c selectors includes: #x:).
		WBMock removeSelector: #x:.
		self deny: (c selectors includes: #x:)]! !

!WBTestSystemChanges methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 03:22'!
testRemoveMutation
	| c |
	c _ self barrierClass.
	self withDefaultMethod: 'x: n x _ n' on: WBMock do:
		[self assert: (c selectors includes: #x:).
		WBMock compile: 'x: n ^ 42'.
		self deny: (c selectors includes: #x:)]! !

!WBTestSystemChanges methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 11:48'!
testClassFormatChangeWithInstance
	| instance instance2 |
	instance _ WBMock new.
	instance2 _ WBMock new.
	barrier add: instance.
	instance y: 42.
	instance2 y: 23.
	WBMock removeInstVarName: 'x'.
	[self assert: instance2 y = 23.
	self assert: instance y = 42]
		ensure: [Object subclass: #WBMock
	instanceVariableNames: 'x y'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WriteBarrier-Tests']! !

!WBTestSystemChanges methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 02:03'!
withDefaultMethod: aString on: aClass do: aBlock
	aClass compile: aString classified: 'tests'.
	aBlock ensure: [aClass compile: aString classified: 'tests']! !

!WBTestSystemChanges methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 03:22'!
testChangeSameField
	| c cm |
	c _ self barrierClass.
	self withDefaultMethod: 'x: n x _ n' on: WBMock do:
		[cm _ c >> #x:.
		WBMock compile: 'x: n x _ n + 1'.
		self assert: (c >> #x:) == cm]! !

!WBTestSystemChanges methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 03:22'!
testSubClass
	| c |
	c _ (barrier builderForClass: WBMockSub) barrierClass.
	self withDefaultMethod: 'y ^ y' on: WBMock do:
		[self deny: (c selectors includes: #y).
		WBMock compile: 'y ^ y ifNil: [y _ 37]' classified: 'accessing'.
		self assert: (c selectors includes: #y)]! !

!WBTestSystemChanges methodsFor: 'as yet unclassified' stamp: 'avi 10/7/2004 03:22'!
testRemoveField
	| c |
	c _ self barrierClass.
	self assert: (c selectors includes: #y:).
	WBMock removeInstVarName: 'y'.
	c _ self barrierClass.
	[self deny: (c selectors includes: #y:)]
		ensure: [WBMock addInstVarName: 'y']! !
WBMethodBuilder initialize!
